import collections
import copy
import datetime
import gc
import time

# import torch
import numpy as np
from sklearn.metrics import mean_absolute_error, mean_squared_error, mean_absolute_percentage_error

from util.logconf import logging
log = logging.getLogger(__name__)
# log.setLevel(logging.WARN)
# log.setLevel(logging.INFO)
log.setLevel(logging.DEBUG)

def evaluate(y_test, y_pred, show=False):
    mse = mean_squared_error(y_test, y_pred, squared=True)
    rmse = mean_squared_error(y_test, y_pred, squared=False)
    mape = mean_absolute_percentage_error(y_test, y_pred) * 100
    # dtw_distance = fastdtw(y_test, y_pred, dist=euclidean)[0] 
    if show:
        print('Model Performance')
        print("Mean squared error = {:0.3f}".format(mse))
        print("Root mean squared error = {:0.3f}".format(rmse))
        print('Mean absolute percentage error = {:0.3f}%.'.format(mape))
        # print('DTW distance = {:0.3f} '.format(dtw_distance))
    else:
        return mse, rmse, mape


def importstr(module_str, from_=None):
    """
    >>> importstr('os')
    <module 'os' from '.../os.pyc'>
    >>> importstr('math', 'fabs')
    <built-in function fabs>
    """
    if from_ is None and ':' in module_str:
        module_str, from_ = module_str.rsplit(':')

    module = __import__(module_str)
    for sub_str in module_str.split('.')[1:]:
        module = getattr(module, sub_str)

    if from_:
        try:
            return getattr(module, from_)
        except:
            raise ImportError('{}.{}'.format(module_str, from_))
    return module

def prhist(ary, prefix_str=None, **kwargs):
    if prefix_str is None:
        prefix_str = ''
    else:
        prefix_str += ' '

    count_ary, bins_ary = np.histogram(ary, **kwargs)
    for i in range(count_ary.shape[0]):
        print("{}{:-8.2f}".format(prefix_str, bins_ary[i]), "{:-10}".format(count_ary[i]))
    print("{}{:-8.2f}".format(prefix_str, bins_ary[-1]))

def enumerateWithEstimate(
        iter,
        desc_str,
        start_ndx=0,
        print_ndx=4,
        backoff=None,
        iter_len=None,
):
    """
    In terms of behavior, `enumerateWithEstimate` is almost identical
    to the standard `enumerate` (the differences are things like how
    our function returns a generator, while `enumerate` returns a
    specialized `<enumerate object at 0x...>`).

    However, the side effects (logging, specifically) are what make the
    function interesting.

    :param iter: `iter` is the iterable that will be passed into
        `enumerate`. Required.

    :param desc_str: This is a human-readable string that describes
        what the loop is doing. The value is arbitrary, but should be
        kept reasonably short. Things like `"epoch 4 training"` or
        `"deleting temp files"` or similar would all make sense.

    :param start_ndx: This parameter defines how many iterations of the
        loop should be skipped before timing actually starts. Skipping
        a few iterations can be useful if there are startup costs like
        caching that are only paid early on, resulting in a skewed
        average when those early iterations dominate the average time
        per iteration.

        NOTE: Using `start_ndx` to skip some iterations makes the time
        spent performing those iterations not be included in the
        displayed duration. Please account for this if you use the
        displayed duration for anything formal.

        This parameter defaults to `0`.

    :param print_ndx: determines which loop interation that the timing
        logging will start on. The intent is that we don't start
        logging until we've given the loop a few iterations to let the
        average time-per-iteration a chance to stablize a bit. We
        require that `print_ndx` not be less than `start_ndx` times
        `backoff`, since `start_ndx` greater than `0` implies that the
        early N iterations are unstable from a timing perspective.

        `print_ndx` defaults to `4`.

    :param backoff: This is used to how many iterations to skip before
        logging again. Frequent logging is less interesting later on,
        so by default we double the gap between logging messages each
        time after the first.

        `backoff` defaults to `2` unless iter_len is > 1000, in which
        case it defaults to `4`.

    :param iter_len: Since we need to know the number of items to
        estimate when the loop will finish, that can be provided by
        passing in a value for `iter_len`. If a value isn't provided,
        then it will be set by using the value of `len(iter)`.

    :return:
    """
    if iter_len is None:
        iter_len = len(iter)

    if backoff is None:
        backoff = 2
        while backoff ** 7 < iter_len:
            backoff *= 2

    assert backoff >= 2
    while print_ndx < start_ndx * backoff:
        print_ndx *= backoff

    log.warning("{} ----/{}, starting".format(
        desc_str,
        iter_len,
    ))
    start_ts = time.time()
    for (current_ndx, item) in enumerate(iter):
        yield (current_ndx, item)
        if current_ndx == print_ndx:
            # ... <1>
            duration_sec = ((time.time() - start_ts)
                            / (current_ndx - start_ndx + 1)
                            * (iter_len-start_ndx)
                            )

            done_dt = datetime.datetime.fromtimestamp(start_ts + duration_sec)
            done_td = datetime.timedelta(seconds=duration_sec)

            log.info("{} {:-4}/{}, done at {}, {}".format(
                desc_str,
                current_ndx,
                iter_len,
                str(done_dt).rsplit('.', 1)[0],
                str(done_td).rsplit('.', 1)[0],
            ))

            print_ndx *= backoff

        if current_ndx + 1 == start_ndx:
            start_ts = time.time()

    log.warning("{} ----/{}, done at {}".format(
        desc_str,
        iter_len,
        str(datetime.datetime.now()).rsplit('.', 1)[0],
    ))











import gzip

from cassandra.cqltypes import BytesType
from diskcache import FanoutCache, Disk,core
from diskcache.core import io
from io import BytesIO
from diskcache.core import MODE_BINARY

from util.logconf import logging
log = logging.getLogger(__name__)
# log.setLevel(logging.WARN)
log.setLevel(logging.INFO)
# log.setLevel(logging.DEBUG)


class GzipDisk(Disk):
    def store(self, value, read, key=None):
        """
        Override from base class diskcache.Disk.

        Chunking is due to needing to work on pythons < 2.7.13:
        - Issue #27130: In the "zlib" module, fix handling of large buffers
          (typically 2 or 4 GiB).  Previously, inputs were limited to 2 GiB, and
          compression and decompression operations did not properly handle results of
          2 or 4 GiB.

        :param value: value to convert
        :param bool read: True when value is file-like object
        :return: (size, mode, filename, value) tuple for Cache table
        """
        # pylint: disable=unidiomatic-typecheck
        if type(value) is BytesType:
            if read:
                value = value.read()
                read = False

            str_io = BytesIO()
            gz_file = gzip.GzipFile(mode='wb', compresslevel=1, fileobj=str_io)

            for offset in range(0, len(value), 2**30):
                gz_file.write(value[offset:offset+2**30])
            gz_file.close()

            value = str_io.getvalue()

        return super(GzipDisk, self).store(value, read)


    def fetch(self, mode, filename, value, read):
        """
        Override from base class diskcache.Disk.

        Chunking is due to needing to work on pythons < 2.7.13:
        - Issue #27130: In the "zlib" module, fix handling of large buffers
          (typically 2 or 4 GiB).  Previously, inputs were limited to 2 GiB, and
          compression and decompression operations did not properly handle results of
          2 or 4 GiB.

        :param int mode: value mode raw, binary, text, or pickle
        :param str filename: filename of corresponding value
        :param value: database value
        :param bool read: when True, return an open file handle
        :return: corresponding Python value
        """
        value = super(GzipDisk, self).fetch(mode, filename, value, read)

        if mode == MODE_BINARY:
            str_io = BytesIO(value)
            gz_file = gzip.GzipFile(mode='rb', fileobj=str_io)
            read_csio = BytesIO()

            while True:
                uncompressed_data = gz_file.read(2**30)
                if uncompressed_data:
                    read_csio.write(uncompressed_data)
                else:
                    break

            value = read_csio.getvalue()

        return value

def getCache(scope_str):
    return FanoutCache('../../data-unversioned/cache/' + scope_str,
                       disk=GzipDisk,
                       shards=64,
                       timeout=1,
                       size_limit=3e11,
                       # disk_min_file_size=2**20,
                       )
















import argparse
import os
import shutil
import sys
import glob
sys.path.append("..")
import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split 
from sklearn.metrics import mean_absolute_error, mean_squared_error, mean_absolute_percentage_error
from probabilistic.GHMM import GHMM
from scipy.spatial.distance import euclidean
# from fastdtw import fastdtw

def evaluate(y_test, y_pred):
    mse = mean_squared_error(y_test, y_pred, squared=True)
    rmse = mean_squared_error(y_test, y_pred, squared=False)
    mape = mean_absolute_percentage_error(y_test, y_pred)
    # dtw_distance = fastdtw(y_test, y_pred, dist=euclidean)[0] 
    print('Model Performance')
    print("Mean squared error = {:0.3f}".format(mse))
    print("Root mean squared error = {:0.3f}".format(rmse))
    print('Mean absolute percentage error = {:0.3f}%.'.format((mape)*100))
    # print('DTW distance = {:0.3f} '.format(dtw_distance))

def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('--stock',
                        help="Stock from S%P",
                        action="store",
                        default=None,
                    )
    parser.add_argument('--forecast',
                        help="Test size training",
                        action="store",
                        default=0.3,
                        type=float,
                    )

    parser.add_argument('--hidden-states',
                        help="Number of hidden states",
                        action="store",
                        default=4,
                        type=int,
                    )

    parser.add_argument('--latency-days',
                        help="Number of latency days",
                        action="store",
                        default=10,
                        type=int,
                    )
    parser.add_argument('--forecasting',
                        help="Number of forecasted days",
                        action="store",
                        default=50,
                        type=int,
                    )

    parser.add_argument('--frac-change',
                        help="Number of latency days",
                        action="store",
                        default=50,
                        type=int,
                    )

    parser.add_argument('--frac-high',
                        help="Number of latency days",
                        action="store",
                        default=10,
                        type=int,
                    )

    parser.add_argument('--frac-low',
                        help="Number of latency days",
                        action="store",
                        default=10,
                        type=int,
                    )

    parser.add_argument('--plot',
                        help="Number of latency days",
                        action="store_true",
                    )
    parser.add_argument('--folder',
                        help="Number of latency days",
                        action="store",
                        default="../../stock/",
                    )     

    cli_args = parser.parse_args()
    stock_predictor = GHMM(hs=cli_args.hidden_states,
                                        ld=cli_args.latency_days,
                                        fc=cli_args.frac_change,
                                        fh=cli_args.frac_high,
                                        fl=cli_args.frac_low)

    df = pd.read_csv(cli_args.folder + cli_args.stock.upper() + '.csv')
    X_train = df[:len(df)-cli_args.forecast]
    X_test = df[-cli_args.forecast:]
    stock_predictor.fit(X_train=X_train)
    y_test = X_test['Close']
    y_pred = stock_predictor.predict(X_test=X_test, plot=cli_args.plot)
    evaluate(y_test, y_pred)

if __name__ == "__main__":
    main()



import argparse
import pandas as pd
import numpy as np
import sys
sys.path.append("..")
from util.StockDataset import StockData

def main():
    parser = argparse.ArgumentParser()

        # Feature implementation:
        # Select to download stocks from different stock-market than S&P

    parser.add_argument('--download',
                        help="download all stocks from S&P",
                        action="store_true",
                        )

    parser.add_argument('--stock',
                        help='Download data from stock market, default is S&P',
                        action='store',
                        default=None,
                        type=str
                        )

    parser.add_argument('--stock-list',
                        help="list of stock to download",
                        action="store",
                        type=str,
                        )

    parser.add_argument('--start',
                        help='In which day start the download, default 2017-01-01',
                        action='store',
                        default=None,
                        type=str
                        )
    parser.add_argument('--end',
                        help='Last day of the dataset, default today',
                        action='store',
                        default=None,
                        type=str
                        )
    parser.add_argument('--update',
                        help="Update the choosen dataset",
                        action='store_true',
                        )
    parser.add_argument('--update-stock',
                        help="Delete a precise stock",
                        action='store',
                        default=None,
                        )
    parser.add_argument('--delete',
                        help="Delete all the folder",
                        action="store_true"
                        )
    parser.add_argument('--delete-stock',
                        help="Delete a precise stock",
                        action='store',
                        default=None
                        )
    parser.add_argument('--range',
                        help="First n stocks in range",
                        action='store',
                        default=0,
                        type=int,
                        )
    parser.add_argument('--folder',
                        help="First n stocks in range",
                        action='store',
                        default="../../stock/",
                        )
   
    cli_args = parser.parse_args()

    if cli_args.stock_list is not None:
        stock_list = [item for item in cli_args.stock_list.split(',')]

    if cli_args.stock_list is not None:
        StockData(download=cli_args.download,
                download_stock=stock_list,
                start=cli_args.start,
                end=cli_args.end,
                update=cli_args.update,
                update_stock=cli_args.update_stock,
                delete=cli_args.delete,
                delete_stock=cli_args.delete_stock,
                range=cli_args.range,
                folder=cli_args.folder)
    else:
        StockData(download=cli_args.download,
                download_stock=cli_args.stock,
                start=cli_args.start,
                end=cli_args.end,
                update=cli_args.update,
                update_stock=cli_args.update_stock,
                delete=cli_args.delete,
                delete_stock=cli_args.delete_stock,
                range=cli_args.range,
                folder=cli_args.folder)     

if __name__ == "__main__":
    main()



import argparse
import os
import shutil
import sys
import glob
sys.path.append("..")
import pandas as pd
import numpy as np
from sklearn.model_selection import train_test_split 
from sklearn.metrics import mean_absolute_error, mean_squared_error, mean_absolute_percentage_error
from neural_network.LSTM import LSTM
from scipy.spatial.distance import euclidean
# from fastdtw import fastdtw
import matplotlib.pyplot as plt

# set style of graphs
plt.style.use('ggplot')
from pylab import rcParams
plt.rcParams['figure.dpi'] = 100

def evaluate(y_test, y_pred):
    mse = mean_squared_error(y_test, y_pred, squared=True)
    rmse = mean_squared_error(y_test, y_pred, squared=False)
    mape = mean_absolute_percentage_error(y_test, y_pred)
    # dtw_distance = fastdtw(y_test, y_pred, dist=euclidean)[0] 
    print('Model Performance')
    print("Mean squared error = {:0.3f}".format(mse))
    print("Root mean squared error = {:0.3f}".format(rmse))
    print('Mean absolute percentage error = {:0.3f}%.'.format((mape)*100))

def main():
    parser = argparse.ArgumentParser()

    parser.add_argument('--stock',
                        help="Stock from S%P",
                        action="store",
                        default=None,
                    )
    parser.add_argument('--test-size',
                        help="Test size training",
                        action="store",
                        default=0.3,
                        type=float,
                    )

    parser.add_argument('--hidden-size',
                        help="Number of neural network ",
                        action="store",
                        default=32,
                        type=int,
                    )

    parser.add_argument('--num-layers',
                        help="Number of layers",
                        action="store",
                        default=1,
                        type=int,
                    )
    parser.add_argument('--batch-size',
                        help="Number of forecasted days",
                        action="store",
                        default=16,
                        type=int,
                    )

    parser.add_argument('--epochs',
                        help="Number of latency days",
                        action="store",
                        default=10,
                        type=int,
                    )

    parser.add_argument('--plot',
                        help="Number of latency days",
                        action="store_true",
                    )
    parser.add_argument('--folder',
                        help="Number of latency days",
                        action="store",
                        default="../../stock/",
                    )     

    cli_args = parser.parse_args()
    stock_predictor = LSTM(hidden_size=cli_args.hidden_size,
                           num_layers=cli_args.num_layers)

    data = pd.read_csv(cli_args.folder + cli_args.stock.upper() + '.csv',
                        parse_dates=True, index_col='Date').dropna(how="any")

    X_train, X_test = train_test_split(data, 
                                test_size=cli_args.test_size, 
                                shuffle=False)

    stock_predictor.fit(X_train, 
                        batch_size=cli_args.batch_size, 
                        epochs=cli_args.epochs)

    y_test = X_test['Close']

    y_pred = stock_predictor.predict(x_test=X_test, plot=cli_args.plot)
    evaluate(y_test, y_pred)

if __name__ == "__main__":
    main()
    
    
    
    
    
    
    
    



import setuptools 

long_description="""
## Description
**stockpy** is a Python Machine Learning library to detect relevant trading patterns and make investmen predictions. At the moment it supports the following algorithms:

- Long Short Term Memory (LSTM)
- Bidirectional Long Short Term Memory (BiLSTM)
- Gated Recurrent Unit (GRU)
- Bidirectional Gated Recurrent Unit (BiGRU)
- Multilayer Perceptron (MLP)
- Gaussian Hidden Markov Models (GHMM)
- Bayesian Neural Networks (BNN)
- Deep Markov Model (DMM)
"""

setuptools.setup(
    name='stockpy-learn',
    version='0.1.8',
    author='Silvio Baratto',
    author_email='silvio.baratto22@gmail.com',
    description='Machine Learning library to make stock market prediction',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SilvioBaratto/stockpy",
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        "Operating System :: OS Independent", 
        'Programming Language :: Python :: 3',
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Mathematics",
    ],
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    install_requires=[
        "torch>=1.9.0",
        "pyro-ppl>=1.7.0",
        "numpy>=1.20.0",
    ],
)
